using CommunityToolkit.Maui.Views;
using Model;
using Model.Stub;

namespace Linaris;

public partial class PlaylistPage : ContentPage
{
	private readonly Manager Manager = (Application.Current as App).Manager;

	public PlaylistPage()
	{
		InitializeComponent();
		BindingContext = Manager.CurrentPlaylist;
	}

	private async void EditPlaylist(object sender, EventArgs e)
	{
        await Navigation.PushModalAsync(new EditPlaylistPage());
    }

	void RemoveCustomTitle(object sender, EventArgs e)
	{
		if (sender is MenuItem menuItem)
		{
			if (menuItem.BindingContext is CustomTitle customTitle)
            {
                Manager.GetPlaylistByName(Manager.CurrentPlaylist.Name).RemoveTitle(customTitle);
            }
		}
	}


    private void Play(object sender, EventArgs e)
	{
		if (sender is Button button)
		{
			if (button.BindingContext is Playlist playlist)
			{
				Manager.CurrentPlaylist = playlist;
				Manager.NextTitle();
                ContentView footer = this.FindByName<ContentView>("Footer");
                var musicElement = footer?.FindByName<MediaElement>("music");
				if (musicElement != null)
				{
                    if (Manager.CurrentPlaying == null) return;
                    musicElement.Source = Manager.CurrentPlaying.Path;
                }
            }
        }
	}

	private void RdmPlay(object sender, EventArgs e)
    {
		if (sender is Button button)
		{
			if (button.BindingContext is Playlist playlist)
			{
				Manager.CurrentPlaylist = playlist;
				Manager.Shuffle();
				Manager.NextTitle();
				ContentView footer = this.FindByName<ContentView>("Footer");
                var musicElement = footer?.FindByName<MediaElement>("music");
                if (musicElement != null)
                {
                    if (Manager.CurrentPlaying == null) return;
                    musicElement.Source = Manager.CurrentPlaying.Path;
                }
            }
		}
    }

    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        ContentView footer = this.FindByName<ContentView>("Footer");
        var musicElement = footer?.FindByName<MediaElement>("music");
        if (musicElement != null)
        {
            musicElement.Stop();
        }
    }
}